/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/store/preview.h"


#include "scope/store/i18n.h"
#include "scope/store/service_manager.h"
#include <unity/UnityExceptions.h>
#include <unity/scopes/ColumnLayout.h>
#include <unity/scopes/PreviewReply.h>
#include <unity/scopes/Variant.h>
#include <unity/scopes/VariantBuilder.h>
#include <unity/scopes/ActionMetadata.h>
#include <unity/scopes/Result.h>
#include <QDebug>


namespace usc = unity::scopes;


namespace
{
static usc::PreviewWidgetList
headerWidgets(Libertine::Store::Package const& app)
{
  usc::PreviewWidget header("hdr", "header");
  header.add_attribute_value("title", usc::Variant(app.name));
  header.add_attribute_value("subtitle", usc::Variant(app.summary));
  header.add_attribute_value("mascot", usc::Variant(app.icon));
  header.add_attribute_value("fallback", usc::Variant("image://theme/placeholder-app-icon"));

  usc::VariantBuilder builder;
  builder.add_tuple({{"value",  usc::Variant(app.render_rating())}});
  header.add_attribute_value("attributes", builder.end());

  usc::PreviewWidgetList widgets;
  widgets.push_back(header);

  qDebug() << "Pushed widgets for package:" << QString::fromStdString(app.name);
  return widgets;
}


static usc::PreviewWidget
metadataWidgets(Libertine::Store::Package const& app)
{
  usc::PreviewWidget widget("other_metadata", "table");
  usc::VariantArray values;
  if (!app.publisher.empty())
  {
    values.push_back(usc::Variant{usc::VariantArray{usc::Variant{"Publisher/Creator"}, usc::Variant{app.publisher}}});
  }
  if (!app.website.empty())
  {
    values.push_back(usc::Variant{usc::VariantArray{usc::Variant{"Website"}, usc::Variant{app.website}}});
  }
  if (!app.license.empty())
  {
    values.push_back(usc::Variant{usc::VariantArray{usc::Variant{"License"}, usc::Variant{app.license}}});
  }

  widget.add_attribute_value("values", usc::Variant(values));
  return widget;
}


static usc::PreviewWidget
buttonWidgets(Libertine::Store::Package const&)
{
  usc::PreviewWidget buttons("buttons", "actions");
  usc::VariantBuilder builder;
  builder.add_tuple(
      {
          {"id", usc::Variant("install_xapp")},
          {"label", usc::Variant(_("Install"))},
      });
  buttons.add_attribute_value("actions", builder.end());
  return buttons;
}


static usc::PreviewWidget
descriptionWidgets(Libertine::Store::Package const& app)
{
  usc::PreviewWidget summary("summary", "text");
  summary.add_attribute_value("title", usc::Variant(_("Info")));
  summary.add_attribute_value("text", usc::Variant(app.description));
  return summary;
}


static usc::PreviewWidget
screenshotWidgets(Libertine::Store::Package const& app)
{
  usc::PreviewWidget gallery("screenshots", "gallery");
  usc::VariantArray arr;
  for (auto const& screenshot: app.screenshots)
  {
    arr.push_back(usc::Variant(screenshot));
  }
  gallery.add_attribute_value("sources", usc::Variant(arr));
  return gallery;
}


static usc::PreviewWidgetList
createWidgets(Libertine::Store::Package const& app)
{
  auto widgets = headerWidgets(app);
  widgets.push_back(buttonWidgets(app));
  widgets.push_back(metadataWidgets(app));
  if (!app.screenshots.empty())
  {
    widgets.push_back(screenshotWidgets(app));
  }
  widgets.push_back(descriptionWidgets(app));

  return widgets;
}
}


Libertine::Store::Preview::
Preview(usc::Result const&         result,
        usc::ActionMetadata const& metadata,
        std::shared_ptr<ServiceManager>  const& service)
  : PreviewQueryBase(result, metadata)
  , service_(service)
{
}


void Libertine::Store::Preview::
cancelled()
{
}


void Libertine::Store::Preview::
createSingleColumnLayout(Package const& app)
{
  oneColumn.column.push_back("hdr");
  oneColumn.column.push_back("buttons");
  oneColumn.column.push_back("other_metadata");
  if (!app.screenshots.empty())
  {
    oneColumn.column.push_back("screenshots");
  }
  oneColumn.column.push_back("summary");
}


void Libertine::Store::Preview::
createDualColumnLayout(Package const& app)
{
  twoColumns.column1.push_back("hdr");
  twoColumns.column1.push_back("buttons");
  twoColumns.column1.push_back("summary");
  twoColumns.column2.push_back("other_metadata");
  if (!app.screenshots.empty())
  {
    twoColumns.column2.push_back("screenshots");
  }
}


void Libertine::Store::Preview::
run(usc::PreviewReplyProxy const& reply)
{
  auto app = service_->app_info(QString::fromStdString(result()["id"].get_string()));
  auto widgets = createWidgets(app);

  createSingleColumnLayout(app);
  createDualColumnLayout(app);

  registerLayouts(reply);
  reply->push(widgets);
}


void Libertine::Store::Preview::
registerLayouts(unity::scopes::PreviewReplyProxy const& reply)
{
    usc::ColumnLayout layout1col(1);
    layout1col.add_column(oneColumn.column);

    usc::ColumnLayout layout2col(2);
    layout2col.add_column(twoColumns.column1);
    layout2col.add_column(twoColumns.column2);

    try
    {
        reply->register_layout({layout1col, layout2col});
    }
    catch (unity::LogicException const& e)
    {
        qWarning() << "Failed to register layout:" << QString::fromStdString(e.what());
    }
}
