# Copyright 2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import apt
import glob
import os
import re

def _use_system_gpg():
    """ Configures APT to use the system-wide GPG store.  Always."""
    for key in 'Dir::Etc::Trusted', 'Dir::Etc::TrustedParts':
        apt.apt_pkg.config.set(key, apt.apt_pkg.config.find_file(key))


def _get_system_sources_list():
    """Concatenates all the system apt/sources.list files into a single text buffer."""
    sources_path = os.path.join('/etc', 'apt')
    sources_list = glob.glob(os.path.join(sources_path, 'sources.list.d', '*.list'))
    sources_list.append(os.path.join(sources_path, 'sources.list'))

    sources = ''
    for source in sources_list:
        with open(source) as f:
            sources += f.read()

    return sources


def _write_local_sources_list(rootdir):
    """ Writes out a user-local sources.list file by copying all the systemwide ones. """
    sources = _get_system_sources_list()
    sources_path = os.path.join(rootdir, 'etc', 'apt')
    os.makedirs(sources_path, exist_ok=True)
    srcfile = os.path.join(sources_path, 'sources.list')
    with open(srcfile, 'w') as f:
        f.write(sources)
    return srcfile


def _get_local_apt_cache():
    """ Gets a user-local APT cache. """
    rootdir = os.path.join('/tmp', '.cache', 'libertine', 'apt')
    srcfile = _write_local_sources_list(rootdir)

    apt_cache = apt.Cache(rootdir=rootdir, memonly=True)
    apt.apt_pkg.config.clear("APT::Update::Post-Invoke-Success")
    apt_cache.update(sources_list=srcfile)
    apt_cache.open()

    return apt_cache


def _get_apt_cache(config):
    """ Factory function to get the APT cache obejct. """
    _use_system_gpg()
    if config.use_local_cache:
        return _get_local_apt_cache()
    else:
        return apt.Cache()

class AptCache(object):
    """docstring for AptCache"""
    def __init__(self, config):
        super(AptCache, self).__init__()
        self.config = config

    def search(self, query):
        cache = _get_apt_cache(self.config)
        return {k: cache[k] for k in cache.keys() if re.match(query, k)}
