/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/store/preview.h"
#include "tests/scope/store/mock_service_manager.h"

#include <unity/scopes/ActionMetadata.h>
#include <unity/scopes/PreviewWidget.h>
#include <unity/scopes/testing/MockPreviewReply.h>
#include <unity/scopes/testing/Result.h>
#include <gtest/gtest.h>
#include <gmock/gmock.h>

namespace
{
using namespace Libertine::Store;


static void
verifyHeader(Package const& package, unity::scopes::PreviewWidget const& header)
{
  EXPECT_EQ("hdr", header.id());
  EXPECT_EQ("header", header.widget_type());

  EXPECT_EQ(package.name, header.attribute_values()["title"].get_string());
  EXPECT_EQ(package.summary, header.attribute_values()["subtitle"].get_string());
  EXPECT_EQ(package.icon, header.attribute_values()["mascot"].get_string());

  auto attributes = header.attribute_values()["attributes"].get_array();
  ASSERT_EQ(1, attributes.size());
  EXPECT_EQ(package.render_rating(), attributes[0].get_dict()["value"].get_string());
}


static void
verifyButtons(unity::scopes::PreviewWidget const& buttons)
{
  EXPECT_EQ("buttons", buttons.id());
  EXPECT_EQ("actions", buttons.widget_type());
  auto buttons_actions = buttons.attribute_values()["actions"].get_array();
  ASSERT_EQ(1, buttons_actions.size());
  EXPECT_EQ("install_xapp", buttons_actions[0].get_dict()["id"].get_string());
  EXPECT_EQ("Install", buttons_actions[0].get_dict()["label"].get_string());
}


static void
verifyMetadata(Package const& package, unity::scopes::PreviewWidget const& props)
{
  EXPECT_EQ("other_metadata", props.id());
  EXPECT_EQ("table", props.widget_type());

  auto propValues = props.attribute_values()["values"].get_array();
  ASSERT_EQ(3, propValues.size());
  ASSERT_EQ(2, propValues[0].get_array().size());
  EXPECT_EQ("Publisher/Creator", propValues[0].get_array()[0].get_string());
  EXPECT_EQ(package.publisher, propValues[0].get_array()[1].get_string());
  ASSERT_EQ(2, propValues[1].get_array().size());
  EXPECT_EQ("Website", propValues[1].get_array()[0].get_string());
  EXPECT_EQ(package.website, propValues[1].get_array()[1].get_string());
  ASSERT_EQ(2, propValues[2].get_array().size());
  EXPECT_EQ("License", propValues[2].get_array()[0].get_string());
  EXPECT_EQ(package.license, propValues[2].get_array()[1].get_string());
}


static void
verifyDescription(Package const& package, unity::scopes::PreviewWidget const& description)
{
  EXPECT_EQ("summary", description.id());
  EXPECT_EQ("text", description.widget_type());
  EXPECT_EQ("Info", description.attribute_values()["title"].get_string());
  EXPECT_EQ(package.description, description.attribute_values()["text"].get_string());
}


static void
verifyScreenshots(Package const& package, unity::scopes::PreviewWidget const& screenshots)
{
  EXPECT_EQ("screenshots", screenshots.id());
  EXPECT_EQ("gallery", screenshots.widget_type());

  auto sources = screenshots.attribute_values()["sources"].get_array();
  ASSERT_EQ(package.screenshots.size(), sources.size());

  for (auto i = 0u; i < package.screenshots.size(); ++i)
  {
    EXPECT_EQ(package.screenshots[i], sources[i].get_string());
  }

}


class StorePreviewTest : public testing::Test
{
protected:
  StorePreviewTest()
    : testing::Test()
    , result()
    , metadata("en_US", "phone")
    , list(new unity::scopes::PreviewWidgetList())
    , reply()
    , proxy(&reply, [](unity::scopes::PreviewReply*) {})
    , service(new testing::NiceMock<MockServiceManager>())
  {
  }

  virtual void
  SetUp()
  {
    EXPECT_CALL(reply, push(testing::_)).WillOnce(testing::SaveArg<0>(list.get()));
  }

  void usePackage(Package const& package)
  {
    result["id"] = unity::scopes::Variant(package.id);
    EXPECT_CALL(*service, app_info(QString::fromStdString(package.id))).WillOnce(testing::Return(package));
  }

  unity::scopes::testing::Result result;
  unity::scopes::ActionMetadata metadata;
  std::unique_ptr<unity::scopes::PreviewWidgetList> list;
  testing::NiceMock<unity::scopes::testing::MockPreviewReply> reply;
  unity::scopes::PreviewReplyProxy proxy;
  std::shared_ptr<testing::NiceMock<MockServiceManager> > service;
};


TEST_F(StorePreviewTest, ShowsDetailedInformationForPackage)
{
  Package package;
  package.name = "vim";
  package.summary = "classic text editor";
  package.description = "this or emacs";
  package.icon = "file:///test/file.png";
  package.rating = 4.2;
  package.publisher = "Woody Boyd";
  package.website = "http://canonical.com/";
  package.license = "GPLv8+";
  package.id = "vim.desktop";
  usePackage(package);

  Preview preview(result, metadata, service);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(4, list->size());

  verifyHeader(package, list->front());
  list->pop_front();
  verifyButtons(list->front());
  list->pop_front();
  verifyMetadata(package, list->front());
  list->pop_front();
  verifyDescription(package, list->front());
}


TEST_F(StorePreviewTest, ShowsScreenshotsWhenAvailable)
{
  Package package;
  package.name = "vim";
  package.summary = "classic text editor";
  package.description = "this or emacs";
  package.icon = "file:///test/file.png";
  package.rating = 4.2;
  package.publisher = "Woody Boyd";
  package.website = "http://canonical.com/";
  package.license = "GPLv8+";
  package.id = "vim.desktop";
  package.screenshots = std::vector<std::string>{"file:///some/string/uri.png", "file:///some/other/string/uri.jpg"};
  usePackage(package);

  Preview preview(result, metadata, service);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(5, list->size());

  verifyHeader(package, list->front());
  list->pop_front();
  verifyButtons(list->front());
  list->pop_front();
  verifyMetadata(package, list->front());
  list->pop_front();
  verifyScreenshots(package, list->front());
  list->pop_front();
  verifyDescription(package, list->front());
}
}
